/*
 * 版权所有 (c) 华为技术有限公司 2021-2022
 * 功能描述：通信库对外协议头文件，第三方通信库需实现如下C接口
 *
 */
#ifndef COMMUNICATION_H
#define COMMUNICATION_H

#include <cstddef>
#include <cstdint>
#include <sys/types.h>

#if defined (__cplusplus)
extern "C" {
#endif

enum VmiSocketErrorCode {
    SOCKET_RECV_FAIL_RETRY = -1,   // 接收函数返回失败，可再调接收函数重试
    SOCKET_RECV_FAIL_DISCONN = -2, // 接收函数返回失败，SOCKET断链，不可用
    SOCKET_SEND_FAIL_RETRY = -3,   // 发送函数返回失败，可再调发送函数重试
    SOCKET_SEND_FAIL_DISCONN = -4, // 发送函数返回失败，SOCKET断链，不可用
};

// 收到新连接的回调函数
using OnNewConnectionCallback = void (*)(int connection);

/**
 * @功能描述：注册新连接建立后的通知回调函数
 * @参数 [in] newConnCb：收到新连接回调函数
 * @返回值：成功返回0，失败返回错误码
 */
int RegConnectionCB(OnNewConnectionCallback cb);

/**
 * @功能描述：销毁通信连接
 * @参数 [in] connection： OnNewConnectionCallback返回的连接句柄
 * @返回值：成功返回0，失败返回错误码
 */
int CloseConnection(int connection);

/**
 * @功能描述：把数据全部发送
 * @参数 [in] connection：OnNewConnectionCallback返回的连接句柄
 * @参数 [in] buf：待发送数据地址
 * @参数 [in] len：待发送数据大小
 * @返回值：成功返回len，表示已将所有数据发送（不存在只发送部分数据的情况）；有错误返回对应的错误码 VmiSocketErrorCode；
 */
ssize_t SendFully(int connection, uint8_t *buf, size_t len);

/**
 * @功能描述：接收数据
 * @参数 [in] connection：OnNewConnectionCallback返回的连接句柄
 * @参数 [in] buf：接收数据的缓存地址
 * @参数 [in] len：接收数据的缓存大小
 * @返回值：成功返回实际收到的数据大小，有错误返回对应的错误码 VmiSocketErrorCode
 */
ssize_t Recv(int connection, uint8_t *buf, size_t len);

#if defined (__cplusplus)
}
#endif
#endif
